/*
 * VC820Tool
 * Copyright (c) 2004 by Andreas Schroeder <andreas@a-netz.de>
 *
 * This file is part of VC820Tool.
 *
 * VC820Tool is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * VC820Tool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VC820Tool; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using System;

namespace VC820Tool
{
	/// <summary>
	/// This interface defines the methods that must be provided by a 
	/// class that should process datagrams.
	/// </summary>
	public interface IDatagramProcessor 
	{
		/// <summary>
		/// This is called if a new datagram should be processed.
		/// </summary>
		/// <param name="dgram"></param>
		void OnDatagram(Datagram dgram);

		/// <summary>
		/// The name of this processor as displayed in the gui
		/// </summary>
		string UIName { get; }

		/// <summary>
		/// The name of this processor as used in the config file
		/// </summary>
		string InternalName { get; }

		/// <summary>
		/// The user interface of this processor
		/// </summary>
		UI.BaseForm UserInterface{ get; }
	}
}
