/*
 * VC820Tool
 * Copyright (c) 2004 by Andreas Schroeder <andreas@a-netz.de>
 *
 * This file is part of VC820Tool.
 *
 * VC820Tool is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * VC820Tool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VC820Tool; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.IO;
using System.Globalization;

namespace VC820Tool.Processors
{
	public class FileRecorderForm : UI.BaseForm, IDatagramProcessor
	{
		private System.Windows.Forms.TextBox txtFile;
		private System.Windows.Forms.Button btChooseFile;
		private System.Windows.Forms.SaveFileDialog dlgFileSave;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		private bool recording = false;
		private System.IO.StreamWriter writer;
		private System.Windows.Forms.Label laState;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.CheckBox chkEnableRecording;

		private System.Windows.Forms.ComboBox cmbDecimalSeparator;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.GroupBox gbSettings;
		private System.Windows.Forms.Label label3;

		private NumberFormatInfo numberFormat;

		private const string internalName = "FileRecorder";

		private string DateTimeFmtPath = "Processor/" + internalName + "/DateTimeFormat";
		private System.Windows.Forms.ComboBox cmbDateTimeFormat;
		private System.Windows.Forms.Label label5;
		private string DecimalSeparatorPath = "Processor/" + internalName + "/DecimalSeparator";
		private System.Windows.Forms.Label laRecordCount;

		/// <summary>
		/// Count of records written to file
		/// </summary>
		private int recordCount = 0;

		public FileRecorderForm()
		{
			InitializeComponent();
			cmbDateTimeFormat.Text = Settings.CM.GetString(DateTimeFmtPath, "dd.MM.yyyy hh:mm:ss.ff");

			numberFormat = (NumberFormatInfo) Application.CurrentCulture.NumberFormat.Clone();
			
			cmbDecimalSeparator.Text = Settings.CM.GetString(DecimalSeparatorPath, ",");
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(FileRecorderForm));
			this.txtFile = new System.Windows.Forms.TextBox();
			this.btChooseFile = new System.Windows.Forms.Button();
			this.dlgFileSave = new System.Windows.Forms.SaveFileDialog();
			this.laState = new System.Windows.Forms.Label();
			this.label1 = new System.Windows.Forms.Label();
			this.chkEnableRecording = new System.Windows.Forms.CheckBox();
			this.cmbDecimalSeparator = new System.Windows.Forms.ComboBox();
			this.label2 = new System.Windows.Forms.Label();
			this.gbSettings = new System.Windows.Forms.GroupBox();
			this.label3 = new System.Windows.Forms.Label();
			this.cmbDateTimeFormat = new System.Windows.Forms.ComboBox();
			this.laRecordCount = new System.Windows.Forms.Label();
			this.label5 = new System.Windows.Forms.Label();
			this.gbSettings.SuspendLayout();
			this.SuspendLayout();
			// 
			// txtFile
			// 
			this.txtFile.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.txtFile.Location = new System.Drawing.Point(16, 40);
			this.txtFile.Name = "txtFile";
			this.txtFile.Size = new System.Drawing.Size(264, 20);
			this.txtFile.TabIndex = 2;
			this.txtFile.Text = "";
			// 
			// btChooseFile
			// 
			this.btChooseFile.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btChooseFile.Image = ((System.Drawing.Image)(resources.GetObject("btChooseFile.Image")));
			this.btChooseFile.Location = new System.Drawing.Point(288, 40);
			this.btChooseFile.Name = "btChooseFile";
			this.btChooseFile.Size = new System.Drawing.Size(24, 20);
			this.btChooseFile.TabIndex = 3;
			this.btChooseFile.Click += new System.EventHandler(this.btChooseFile_Click);
			// 
			// dlgFileSave
			// 
			this.dlgFileSave.Filter = "Comma Separated Values (*.csv)|*.csv|All Files|*.*";
			// 
			// laState
			// 
			this.laState.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.laState.BackColor = System.Drawing.Color.Red;
			this.laState.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.laState.Location = new System.Drawing.Point(144, 16);
			this.laState.Name = "laState";
			this.laState.Size = new System.Drawing.Size(88, 16);
			this.laState.TabIndex = 6;
			this.laState.Text = "Not Recording";
			this.laState.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(24, 24);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(40, 16);
			this.label1.TabIndex = 7;
			this.label1.Text = "File";
			// 
			// chkEnableRecording
			// 
			this.chkEnableRecording.Location = new System.Drawing.Point(16, 16);
			this.chkEnableRecording.Name = "chkEnableRecording";
			this.chkEnableRecording.Size = new System.Drawing.Size(112, 16);
			this.chkEnableRecording.TabIndex = 8;
			this.chkEnableRecording.Text = "Recording enabled";
			this.chkEnableRecording.CheckedChanged += new System.EventHandler(this.chkEnableRecording_CheckedChanged);
			// 
			// cmbDecimalSeparator
			// 
			this.cmbDecimalSeparator.Items.AddRange(new object[] {
																	 ",",
																	 "."});
			this.cmbDecimalSeparator.Location = new System.Drawing.Point(16, 88);
			this.cmbDecimalSeparator.Name = "cmbDecimalSeparator";
			this.cmbDecimalSeparator.Size = new System.Drawing.Size(121, 21);
			this.cmbDecimalSeparator.TabIndex = 9;
			this.cmbDecimalSeparator.SelectedIndexChanged += new System.EventHandler(this.cmbDecimalSeparator_SelectedIndexChanged);
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(24, 72);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(100, 16);
			this.label2.TabIndex = 10;
			this.label2.Text = "Decimal Separator";
			// 
			// gbSettings
			// 
			this.gbSettings.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.gbSettings.Controls.Add(this.label3);
			this.gbSettings.Controls.Add(this.cmbDateTimeFormat);
			this.gbSettings.Controls.Add(this.label2);
			this.gbSettings.Controls.Add(this.cmbDecimalSeparator);
			this.gbSettings.Controls.Add(this.btChooseFile);
			this.gbSettings.Controls.Add(this.txtFile);
			this.gbSettings.Controls.Add(this.label1);
			this.gbSettings.Location = new System.Drawing.Point(8, 64);
			this.gbSettings.Name = "gbSettings";
			this.gbSettings.Size = new System.Drawing.Size(328, 128);
			this.gbSettings.TabIndex = 11;
			this.gbSettings.TabStop = false;
			this.gbSettings.Text = "Settings";
			// 
			// label3
			// 
			this.label3.Location = new System.Drawing.Point(168, 72);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(112, 16);
			this.label3.TabIndex = 12;
			this.label3.Text = "Date / Time Format";
			// 
			// cmbDateTimeFormat
			// 
			this.cmbDateTimeFormat.Items.AddRange(new object[] {
																   "dd.MM.yyyy hh:mm:ss.ff",
																   "hh:mm:ss.ff"});
			this.cmbDateTimeFormat.Location = new System.Drawing.Point(152, 88);
			this.cmbDateTimeFormat.Name = "cmbDateTimeFormat";
			this.cmbDateTimeFormat.Size = new System.Drawing.Size(160, 21);
			this.cmbDateTimeFormat.TabIndex = 11;
			this.cmbDateTimeFormat.SelectedIndexChanged += new System.EventHandler(this.cmbTimeFormat_SelectedIndexChanged);
			// 
			// laRecordCount
			// 
			this.laRecordCount.BackColor = System.Drawing.SystemColors.Window;
			this.laRecordCount.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.laRecordCount.Location = new System.Drawing.Point(144, 40);
			this.laRecordCount.Name = "laRecordCount";
			this.laRecordCount.Size = new System.Drawing.Size(88, 16);
			this.laRecordCount.TabIndex = 12;
			this.laRecordCount.Text = "0";
			this.laRecordCount.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// label5
			// 
			this.label5.Location = new System.Drawing.Point(32, 40);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(88, 16);
			this.label5.TabIndex = 13;
			this.label5.Text = "Record Count";
			// 
			// FileRecorderForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(346, 199);
			this.Controls.Add(this.label5);
			this.Controls.Add(this.laRecordCount);
			this.Controls.Add(this.gbSettings);
			this.Controls.Add(this.chkEnableRecording);
			this.Controls.Add(this.laState);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.MinimumSize = new System.Drawing.Size(328, 200);
			this.Name = "FileRecorderForm";
			this.Text = "File Recoder";
			this.gbSettings.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		private void btChooseFile_Click(object sender, System.EventArgs e)
		{
			dlgFileSave.Title = "Choose Recoder File";
			dlgFileSave.OverwritePrompt = false;

			DialogResult result = dlgFileSave.ShowDialog(this);

			if(result == DialogResult.OK)
			{
				txtFile.Text = dlgFileSave.FileName;
			}
		}
		
		private void SetRecordingState(bool bRecord)
		{
			if(bRecord)
			{
				laState.Text = "Recording";
				laState.BackColor = Color.Green;

				IsRecording = true;
			}
			else
			{
				laState.Text = "Not Recording";
				laState.BackColor = Color.Red;
				
				IsRecording = false;
			}
		}

		/// <summary>
		/// Write the datagram to the already opened writer object
		/// </summary>
		/// <param name="dgram"></param>
		public void WriteToFile(Datagram dgram)
		{
			string number = string.Format(numberFormat, "{0}", dgram.RealValue);

			writer.WriteLine("\"{0}\";{1};\"{2}\"",
				number,
				Datagram.GetUnitName(dgram.Unit), 
				dgram.Timestamp.ToString(cmbDateTimeFormat.Text));

			RecordCount++;
		}

		private int RecordCount
		{
			get
			{
				return recordCount;
			}
			set
			{
				recordCount = value;
				laRecordCount.Text = recordCount.ToString();
			}
		}

		/// <summary>
		/// Handles the CheckedChanged event of the EnableRecording button, 
		/// changes the recording state of the file recorder
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void chkEnableRecording_CheckedChanged(object sender, System.EventArgs e)
		{
			gbSettings.Enabled = !chkEnableRecording.Checked;

			if(chkEnableRecording.Checked)
			{
				RecordCount = 0;

				try
				{	
					writer = new StreamWriter(txtFile.Text, 
						true, 
						System.Text.Encoding.Default);

					SetRecordingState( true );
				}
				catch(Exception ex)
				{
					chkEnableRecording.Checked = false;
					MessageBox.Show(this,
						"Failed to open recording file: " + ex.Message,
						Settings.AppTitle,
						MessageBoxButtons.OK,
						MessageBoxIcon.Exclamation);
				}
			}
			else
			{
				SetRecordingState( false );

				try
				{
					writer.Close();
				}
				catch(Exception ex)
				{
					System.Diagnostics.Debug.WriteLine("Failed to close recording file: " + ex.Message);
				}
			}
		}

		/// <summary>
		/// True, wenn writing data to the file
		/// </summary>
		private bool IsRecording
		{
			get
			{
				return recording;
			}
			set
			{
				recording = value;
			}
		}

		private void cmbDecimalSeparator_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			numberFormat.NumberDecimalSeparator = (string) cmbDecimalSeparator.SelectedItem;
			Settings.CM.SetString(DecimalSeparatorPath, cmbDecimalSeparator.Text);
		}

		private void cmbTimeFormat_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			Settings.CM.SetString(DateTimeFmtPath, cmbDateTimeFormat.Text);
		}

		#region Implementation of IDatagramProcessor

		/// <summary>
		/// Writes the datagram to the selected file if IsRecording is set to true
		/// </summary>
		/// <param name="dgram"></param>
		public void OnDatagram(Datagram dgram)
		{
			if(IsRecording) 
			{
				WriteToFile(dgram);
			}
		}

		public string UIName
		{
			get
			{
				return "File Recorder";
			}
		}

		public string InternalName
		{
			get
			{
				return internalName;
			}
		}
		
		public UI.BaseForm UserInterface
		{
			get
			{
				return this;
			}
		}
		#endregion
	}
}
