/*
 * VC820Tool
 * Copyright (c) 2003 by Andreas Schroeder <andreas@a-netz.de>
 *
 * This file is part of VC820Tool.
 *
 * VC820Tool is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * VC820Tool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VC820Tool; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using SharpUtils;

namespace VC820Tool.Processors
{
	/// <summary>
	/// Summary description for GraphForm.
	/// </summary>
	public class GraphForm : UI.BaseForm, IDatagramProcessor
	{
		private System.Windows.Forms.CheckBox chkMarkPoints;
		private SimpleGraph graphData;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.Button btApply;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.NumericUpDown nudBufferLength;

		private SharpUtils.MaxLengthQueue queue;
		
		public GraphForm()
		{
			InitializeComponent();
			queue = new MaxLengthQueue(Settings.CM.GetInt("Processor/" + this.InternalName + "/BufferLength", 20));
			graphData.Buffer = queue;

			nudBufferLength.Value = queue.Length;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.chkMarkPoints = new System.Windows.Forms.CheckBox();
			this.graphData = new Processors.SimpleGraph();
			this.btApply = new System.Windows.Forms.Button();
			this.nudBufferLength = new System.Windows.Forms.NumericUpDown();
			this.label1 = new System.Windows.Forms.Label();
			((System.ComponentModel.ISupportInitialize)(this.nudBufferLength)).BeginInit();
			this.SuspendLayout();
			// 
			// chkMarkPoints
			// 
			this.chkMarkPoints.Location = new System.Drawing.Point(16, 8);
			this.chkMarkPoints.Name = "chkMarkPoints";
			this.chkMarkPoints.Size = new System.Drawing.Size(104, 16);
			this.chkMarkPoints.TabIndex = 19;
			this.chkMarkPoints.Text = "Mark Points";
			this.chkMarkPoints.CheckedChanged += new System.EventHandler(this.chkMarkPoints_CheckedChanged);
			// 
			// graphData
			// 
			this.graphData.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right);
			this.graphData.Buffer = null;
			this.graphData.Location = new System.Drawing.Point(8, 32);
			this.graphData.MarkPoints = false;
			this.graphData.Name = "graphData";
			this.graphData.Size = new System.Drawing.Size(464, 264);
			this.graphData.TabIndex = 18;
			// 
			// btApply
			// 
			this.btApply.Location = new System.Drawing.Point(280, 8);
			this.btApply.Name = "btApply";
			this.btApply.Size = new System.Drawing.Size(56, 20);
			this.btApply.TabIndex = 21;
			this.btApply.Text = "Apply";
			this.btApply.Click += new System.EventHandler(this.btApply_Click);
			// 
			// nudBufferLength
			// 
			this.nudBufferLength.Location = new System.Drawing.Point(208, 8);
			this.nudBufferLength.Maximum = new System.Decimal(new int[] {
																			500,
																			0,
																			0,
																			0});
			this.nudBufferLength.Minimum = new System.Decimal(new int[] {
																			1,
																			0,
																			0,
																			0});
			this.nudBufferLength.Name = "nudBufferLength";
			this.nudBufferLength.Size = new System.Drawing.Size(64, 20);
			this.nudBufferLength.TabIndex = 22;
			this.nudBufferLength.Value = new System.Decimal(new int[] {
																		  1,
																		  0,
																		  0,
																		  0});
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(136, 10);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(72, 16);
			this.label1.TabIndex = 23;
			this.label1.Text = "Buffer Length";
			// 
			// GraphForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(480, 301);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.label1,
																		  this.nudBufferLength,
																		  this.btApply,
																		  this.chkMarkPoints,
																		  this.graphData});
			this.Name = "GraphForm";
			this.Text = "Graph";
			((System.ComponentModel.ISupportInitialize)(this.nudBufferLength)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		private void chkMarkPoints_CheckedChanged(object sender, System.EventArgs e)
		{
			graphData.MarkPoints = chkMarkPoints.Checked;
		}

		#region Implementation of IDatagramProcessor
		public void OnDatagram(Datagram dgram)
		{
			queue.Enqueue(dgram);

			graphData.Refresh();
		}

		private void btApply_Click(object sender, System.EventArgs e)
		{
			queue.Length = (int) nudBufferLength.Value;
		}

		public string UIName
		{
			get
			{
				return "Graph";
			}
		}

		public string InternalName
		{
			get
			{
				return "Graph";
			}
		}

		public UI.BaseForm UserInterface
		{
			get
			{
				return this;
			}
		}
		#endregion


	}
}
