/*
 * VC820Tool
 * Copyright (c) 2004 by Andreas Schroeder <andreas@a-netz.de>
 *
 * This file is part of VC820Tool.
 *
 * VC820Tool is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * VC820Tool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VC820Tool; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using SharpUtils;

namespace VC820Tool.Processors
{
	/// <summary>
	/// Summary description for SimpleGraph.
	/// </summary>
	public class SimpleGraph : System.Windows.Forms.UserControl
	{
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.Panel pnlGraph;

		private MaxLengthQueue buffer;

		private decimal minimum;
		private System.Windows.Forms.Label laMin;
		private System.Windows.Forms.Label laMax;
		private decimal maximum;

		private string sSciNumberFmt;
		private System.Windows.Forms.Label laValueCount;
		private bool markpoints;

		public SimpleGraph()
		{
			InitializeComponent();

			sSciNumberFmt = Settings.CM.GetString("/StringFormat/NumScience", "0.000E00");
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.pnlGraph = new System.Windows.Forms.Panel();
			this.laMin = new System.Windows.Forms.Label();
			this.laMax = new System.Windows.Forms.Label();
			this.laValueCount = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// pnlGraph
			// 
			this.pnlGraph.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right);
			this.pnlGraph.BackColor = System.Drawing.Color.White;
			this.pnlGraph.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.pnlGraph.Location = new System.Drawing.Point(88, 8);
			this.pnlGraph.Name = "pnlGraph";
			this.pnlGraph.Size = new System.Drawing.Size(168, 120);
			this.pnlGraph.TabIndex = 0;
			this.pnlGraph.Resize += new System.EventHandler(this.pnlGraph_Resize);
			this.pnlGraph.Paint += new System.Windows.Forms.PaintEventHandler(this.pnlGraph_Paint);
			// 
			// laMin
			// 
			this.laMin.Anchor = (System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left);
			this.laMin.BackColor = System.Drawing.Color.White;
			this.laMin.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.laMin.Location = new System.Drawing.Point(8, 112);
			this.laMin.Name = "laMin";
			this.laMin.Size = new System.Drawing.Size(72, 16);
			this.laMin.TabIndex = 3;
			this.laMin.Text = "0";
			this.laMin.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// laMax
			// 
			this.laMax.BackColor = System.Drawing.Color.White;
			this.laMax.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.laMax.Location = new System.Drawing.Point(8, 8);
			this.laMax.Name = "laMax";
			this.laMax.Size = new System.Drawing.Size(72, 16);
			this.laMax.TabIndex = 4;
			this.laMax.Text = "0";
			this.laMax.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// laValueCount
			// 
			this.laValueCount.Anchor = (System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right);
			this.laValueCount.BackColor = System.Drawing.Color.White;
			this.laValueCount.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.laValueCount.Location = new System.Drawing.Point(176, 136);
			this.laValueCount.Name = "laValueCount";
			this.laValueCount.Size = new System.Drawing.Size(72, 16);
			this.laValueCount.TabIndex = 5;
			this.laValueCount.Text = "0";
			this.laValueCount.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// SimpleGraph
			// 
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.laValueCount,
																		  this.laMax,
																		  this.laMin,
																		  this.pnlGraph});
			this.Name = "SimpleGraph";
			this.Size = new System.Drawing.Size(264, 160);
			this.ResumeLayout(false);

		}
		#endregion

		private int MapDataToGraph(decimal data, decimal maxgraph)
		{
			return (int) ( (1 - (data - minimum) / ((maximum - minimum + 1e-5M)) ) * maxgraph);
		}

		private void pnlGraph_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			try
			{
				if(buffer==null || buffer.Count < 2)
				{
					return;
				}

				CalculateExtremum();

				decimal interval = ((decimal)pnlGraph.Width) / ((decimal)buffer.Count);
				decimal xpos = 0;
				int ypos = 0;
				Point[] points = new Point[buffer.Count];
				int i = 0;

				foreach(Datagram dgram in buffer.List)
				{
					ypos = MapDataToGraph(dgram.RealValue, (decimal) pnlGraph.Height - 10) + 5;

					points[i++] = new Point(((int)xpos), ypos);
					xpos += interval;
				}
				
				e.Graphics.DrawLines(System.Drawing.Pens.Black, points);			

				if(markpoints)
				{
					DrawPointMarks(points, e.Graphics);
				}

				laValueCount.Text = points.Length.ToString();
			}
			catch(Exception ex)
			{
				System.Diagnostics.Debug.WriteLine(ex.Message);
			}
		}

		private void DrawPointMarks(Point[] points, Graphics g)
		{
			foreach(Point p in points)
			{
				g.DrawLine(Pens.Blue,
					p.X - 1,
					p.Y - 1,
					p.X + 1,
					p.Y + 1);
				g.DrawLine(Pens.Blue,
					p.X + 1,
					p.Y - 1,
					p.X - 1,
					p.Y + 1);
			}
		}

		private void CalculateExtremum()
		{
			bool firstValue = true;

			foreach(Datagram d in buffer.List)
			{
				if(firstValue)
				{
					minimum = d.RealValue;
					maximum = d.RealValue;
					firstValue = false;
				}
				else
				{
					minimum = minimum > d.RealValue ? d.RealValue : minimum;
					maximum = maximum < d.RealValue ? d.RealValue : maximum;
				}
			}

			laMin.Text = minimum.ToString(sSciNumberFmt);
			laMax.Text = maximum.ToString(sSciNumberFmt);
		}

		private void pnlGraph_Resize(object sender, System.EventArgs e)
		{
			this.Refresh();
		}

		public MaxLengthQueue Buffer
		{
			get
			{
				return buffer;
			}
			set
			{
				buffer = value;
			}
		}

		public bool MarkPoints
		{
			get
			{
				return markpoints;
			}
			set
			{
				markpoints = value;
			}
		}
	}
}
