/*
 * VC820Tool
 * Copyright (c) 2004 by Andreas Schroeder <andreas@a-netz.de>
 *
 * This file is part of VC820Tool.
 *
 * VC820Tool is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * VC820Tool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VC820Tool; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

namespace VC820Tool.Processors
{
	/// <summary>
	/// Summary description for StatisticsForm.
	/// </summary>
	public class StatisticsForm : UI.BaseForm, IDatagramProcessor
	{
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.Label laAverage;
		private System.Windows.Forms.Label laTendency;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.Button btApply;
		private System.Windows.Forms.NumericUpDown nudBuffer;
		private System.Windows.Forms.Label laBufferCount;
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.Label label4;
		private System.Windows.Forms.Label label6;
		private System.Windows.Forms.Label laMaximum;
		private System.Windows.Forms.Label laMinimum;
		private System.Windows.Forms.Button btReset;
		private System.Windows.Forms.GroupBox groupBox2;
		private Statistics statistics;
		private System.Windows.Forms.GroupBox groupBox3;
		private System.Windows.Forms.GroupBox groupBox4;
		private System.Windows.Forms.GroupBox groupBox5;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.Label laStdDev;

		private string sSciNumFmt;

		private const string internalName = "Statistics";

		private const string SciNumberFormatPath = "/Processor/" + internalName + "/SciNumberFormat";
		private const string BufferLengthPath = "/Processor/" + internalName + "/BufferLength";
		
		public StatisticsForm()
		{
			InitializeComponent();

			nudBuffer.Value = Settings.CM.GetInt(BufferLengthPath, 25);
			statistics = new Statistics((int)nudBuffer.Value);

			sSciNumFmt = Settings.CM.GetString(SciNumberFormatPath, "0.000e00");
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.laTendency = new System.Windows.Forms.Label();
			this.laAverage = new System.Windows.Forms.Label();
			this.label3 = new System.Windows.Forms.Label();
			this.nudBuffer = new System.Windows.Forms.NumericUpDown();
			this.btApply = new System.Windows.Forms.Button();
			this.laBufferCount = new System.Windows.Forms.Label();
			this.label5 = new System.Windows.Forms.Label();
			this.label4 = new System.Windows.Forms.Label();
			this.label6 = new System.Windows.Forms.Label();
			this.laMaximum = new System.Windows.Forms.Label();
			this.laMinimum = new System.Windows.Forms.Label();
			this.btReset = new System.Windows.Forms.Button();
			this.groupBox2 = new System.Windows.Forms.GroupBox();
			this.groupBox3 = new System.Windows.Forms.GroupBox();
			this.groupBox4 = new System.Windows.Forms.GroupBox();
			this.groupBox5 = new System.Windows.Forms.GroupBox();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.laStdDev = new System.Windows.Forms.Label();
			((System.ComponentModel.ISupportInitialize)(this.nudBuffer)).BeginInit();
			this.groupBox2.SuspendLayout();
			this.groupBox3.SuspendLayout();
			this.groupBox4.SuspendLayout();
			this.groupBox5.SuspendLayout();
			this.groupBox1.SuspendLayout();
			this.SuspendLayout();
			// 
			// laTendency
			// 
			this.laTendency.BackColor = System.Drawing.Color.White;
			this.laTendency.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.laTendency.Location = new System.Drawing.Point(16, 24);
			this.laTendency.Name = "laTendency";
			this.laTendency.Size = new System.Drawing.Size(120, 16);
			this.laTendency.TabIndex = 0;
			this.laTendency.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// laAverage
			// 
			this.laAverage.BackColor = System.Drawing.Color.White;
			this.laAverage.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.laAverage.Location = new System.Drawing.Point(16, 24);
			this.laAverage.Name = "laAverage";
			this.laAverage.Size = new System.Drawing.Size(120, 16);
			this.laAverage.TabIndex = 3;
			this.laAverage.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// label3
			// 
			this.label3.Location = new System.Drawing.Point(16, 64);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(100, 16);
			this.label3.TabIndex = 4;
			this.label3.Text = "Max. Puffer-Lnge";
			// 
			// nudBuffer
			// 
			this.nudBuffer.Location = new System.Drawing.Point(16, 80);
			this.nudBuffer.Maximum = new System.Decimal(new int[] {
																	  500,
																	  0,
																	  0,
																	  0});
			this.nudBuffer.Minimum = new System.Decimal(new int[] {
																	  1,
																	  0,
																	  0,
																	  0});
			this.nudBuffer.Name = "nudBuffer";
			this.nudBuffer.TabIndex = 5;
			this.nudBuffer.Value = new System.Decimal(new int[] {
																	1,
																	0,
																	0,
																	0});
			// 
			// btApply
			// 
			this.btApply.Location = new System.Drawing.Point(80, 112);
			this.btApply.Name = "btApply";
			this.btApply.Size = new System.Drawing.Size(56, 20);
			this.btApply.TabIndex = 6;
			this.btApply.Text = "Apply";
			this.btApply.Click += new System.EventHandler(this.btApply_Click);
			// 
			// laBufferCount
			// 
			this.laBufferCount.BackColor = System.Drawing.Color.White;
			this.laBufferCount.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.laBufferCount.Location = new System.Drawing.Point(16, 40);
			this.laBufferCount.Name = "laBufferCount";
			this.laBufferCount.Size = new System.Drawing.Size(120, 16);
			this.laBufferCount.TabIndex = 7;
			this.laBufferCount.Text = "0";
			// 
			// label5
			// 
			this.label5.Location = new System.Drawing.Point(16, 24);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(88, 16);
			this.label5.TabIndex = 8;
			this.label5.Text = "Puffer-Fllstand";
			// 
			// label4
			// 
			this.label4.Location = new System.Drawing.Point(16, 64);
			this.label4.Name = "label4";
			this.label4.Size = new System.Drawing.Size(56, 16);
			this.label4.TabIndex = 9;
			this.label4.Text = "Maximum";
			// 
			// label6
			// 
			this.label6.Location = new System.Drawing.Point(16, 24);
			this.label6.Name = "label6";
			this.label6.Size = new System.Drawing.Size(56, 16);
			this.label6.TabIndex = 10;
			this.label6.Text = "Minimum";
			// 
			// laMaximum
			// 
			this.laMaximum.BackColor = System.Drawing.Color.White;
			this.laMaximum.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.laMaximum.Location = new System.Drawing.Point(16, 80);
			this.laMaximum.Name = "laMaximum";
			this.laMaximum.Size = new System.Drawing.Size(120, 16);
			this.laMaximum.TabIndex = 11;
			this.laMaximum.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// laMinimum
			// 
			this.laMinimum.BackColor = System.Drawing.Color.White;
			this.laMinimum.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.laMinimum.Location = new System.Drawing.Point(16, 40);
			this.laMinimum.Name = "laMinimum";
			this.laMinimum.Size = new System.Drawing.Size(120, 16);
			this.laMinimum.TabIndex = 12;
			this.laMinimum.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// btReset
			// 
			this.btReset.Location = new System.Drawing.Point(88, 112);
			this.btReset.Name = "btReset";
			this.btReset.Size = new System.Drawing.Size(48, 20);
			this.btReset.TabIndex = 13;
			this.btReset.Text = "Reset";
			this.btReset.Click += new System.EventHandler(this.btReset_Click);
			// 
			// groupBox2
			// 
			this.groupBox2.Controls.Add(this.btApply);
			this.groupBox2.Controls.Add(this.label3);
			this.groupBox2.Controls.Add(this.nudBuffer);
			this.groupBox2.Controls.Add(this.label5);
			this.groupBox2.Controls.Add(this.laBufferCount);
			this.groupBox2.Location = new System.Drawing.Point(8, 80);
			this.groupBox2.Name = "groupBox2";
			this.groupBox2.Size = new System.Drawing.Size(152, 144);
			this.groupBox2.TabIndex = 15;
			this.groupBox2.TabStop = false;
			this.groupBox2.Text = "Buffer";
			// 
			// groupBox3
			// 
			this.groupBox3.Controls.Add(this.laTendency);
			this.groupBox3.Location = new System.Drawing.Point(176, 8);
			this.groupBox3.Name = "groupBox3";
			this.groupBox3.Size = new System.Drawing.Size(152, 56);
			this.groupBox3.TabIndex = 16;
			this.groupBox3.TabStop = false;
			this.groupBox3.Text = "Tendency";
			// 
			// groupBox4
			// 
			this.groupBox4.Controls.Add(this.label6);
			this.groupBox4.Controls.Add(this.laMinimum);
			this.groupBox4.Controls.Add(this.laMaximum);
			this.groupBox4.Controls.Add(this.label4);
			this.groupBox4.Controls.Add(this.btReset);
			this.groupBox4.Location = new System.Drawing.Point(176, 80);
			this.groupBox4.Name = "groupBox4";
			this.groupBox4.Size = new System.Drawing.Size(152, 144);
			this.groupBox4.TabIndex = 17;
			this.groupBox4.TabStop = false;
			this.groupBox4.Text = "Extrema";
			// 
			// groupBox5
			// 
			this.groupBox5.Controls.Add(this.laAverage);
			this.groupBox5.Location = new System.Drawing.Point(8, 8);
			this.groupBox5.Name = "groupBox5";
			this.groupBox5.Size = new System.Drawing.Size(152, 56);
			this.groupBox5.TabIndex = 18;
			this.groupBox5.TabStop = false;
			this.groupBox5.Text = "Average";
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.Add(this.laStdDev);
			this.groupBox1.Location = new System.Drawing.Point(8, 232);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(152, 56);
			this.groupBox1.TabIndex = 19;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Standard Deviation";
			// 
			// laStdDev
			// 
			this.laStdDev.BackColor = System.Drawing.Color.White;
			this.laStdDev.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.laStdDev.Location = new System.Drawing.Point(16, 24);
			this.laStdDev.Name = "laStdDev";
			this.laStdDev.Size = new System.Drawing.Size(120, 16);
			this.laStdDev.TabIndex = 0;
			// 
			// StatisticsForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(338, 303);
			this.Controls.Add(this.groupBox1);
			this.Controls.Add(this.groupBox5);
			this.Controls.Add(this.groupBox4);
			this.Controls.Add(this.groupBox3);
			this.Controls.Add(this.groupBox2);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "StatisticsForm";
			this.Text = "Statistics";
			((System.ComponentModel.ISupportInitialize)(this.nudBuffer)).EndInit();
			this.groupBox2.ResumeLayout(false);
			this.groupBox3.ResumeLayout(false);
			this.groupBox4.ResumeLayout(false);
			this.groupBox5.ResumeLayout(false);
			this.groupBox1.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		private void btApply_Click(object sender, System.EventArgs e)
		{
			statistics.BufferLength = (int) nudBuffer.Value;
			Settings.CM.SetString(BufferLengthPath, nudBuffer.Value.ToString());
		}

		private void btReset_Click(object sender, System.EventArgs e)
		{
			statistics.ResetMinMax();
		}

		#region Implementation of IDatagramProcessor
		public void OnDatagram(Datagram dgram)
		{
			statistics.OnNewDatagram(dgram);
			laAverage.Text = statistics.ArithmethicAverage.ToString(sSciNumFmt); 

			switch(statistics.Tendency)
			{
				case Statistics.TendencyEnum.Falling:
					laTendency.Text = "Falling";
					break;
				case Statistics.TendencyEnum.Rising:
					laTendency.Text = "Rising";
					break;
				case Statistics.TendencyEnum.Equal:
					laTendency.Text = "Equal";
					break;
			}

			laBufferCount.Text = statistics.BufferCount.ToString();

			laMinimum.Text = statistics.Minimum.ToString(sSciNumFmt);
			laMaximum.Text = statistics.Maximum.ToString(sSciNumFmt);
			laStdDev.Text = statistics.StandardDeviation.ToString(sSciNumFmt);
		}

		public string UIName
		{
			get
			{
				return "Statistics";
			}
		}

		public string InternalName
		{
			get
			{
				return "Statistics";
			}
		}

		public UI.BaseForm UserInterface
		{
			get
			{
				return this;
			}
		}
		#endregion
	}
}
