/*
 * VC820Tool
 * Copyright (c) 2004 by Andreas Schroeder <andreas@a-netz.de>
 *
 * This file is part of VC820Tool.
 *
 * VC820Tool is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * VC820Tool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VC820Tool; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;

namespace VC820Tool.Processors
{
	public class UDPNetworkProcessor : UI.BaseForm, IDatagramProcessor
	{
		private System.ComponentModel.Container components = null;
		private int udpport;
		private UdpClient udpclient;
		private System.Windows.Forms.Button btApply;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.GroupBox groupBox2;
		private System.Windows.Forms.TextBox txtHost;
		private System.Windows.Forms.RadioButton rbAddress;
		private System.Windows.Forms.RadioButton rbBroadcast;
		private System.Windows.Forms.NumericUpDown nudPort;
		private IPEndPoint destip;

		public UDPNetworkProcessor()
		{
			InitializeComponent();

			udpport = Settings.CM.GetInt("Processor/" + this.InternalName + "/UDPPort", 5454);

			destip = new IPEndPoint(IPAddress.Broadcast, udpport);

			udpclient = new UdpClient();

			nudPort.Value = udpport;

			txtHost.Text = Settings.CM.GetString("Processor/" + this.InternalName + "/DestHost", "Localhost");
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.btApply = new System.Windows.Forms.Button();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.groupBox2 = new System.Windows.Forms.GroupBox();
			this.txtHost = new System.Windows.Forms.TextBox();
			this.rbAddress = new System.Windows.Forms.RadioButton();
			this.rbBroadcast = new System.Windows.Forms.RadioButton();
			this.nudPort = new System.Windows.Forms.NumericUpDown();
			this.groupBox1.SuspendLayout();
			this.groupBox2.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.nudPort)).BeginInit();
			this.SuspendLayout();
			// 
			// btApply
			// 
			this.btApply.Location = new System.Drawing.Point(104, 168);
			this.btApply.Name = "btApply";
			this.btApply.Size = new System.Drawing.Size(56, 20);
			this.btApply.TabIndex = 3;
			this.btApply.Text = "Apply";
			this.btApply.Click += new System.EventHandler(this.btApply_Click);
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.nudPort});
			this.groupBox1.Location = new System.Drawing.Point(8, 104);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(160, 56);
			this.groupBox1.TabIndex = 4;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Port";
			// 
			// groupBox2
			// 
			this.groupBox2.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.txtHost,
																					this.rbAddress,
																					this.rbBroadcast});
			this.groupBox2.Location = new System.Drawing.Point(8, 8);
			this.groupBox2.Name = "groupBox2";
			this.groupBox2.Size = new System.Drawing.Size(160, 88);
			this.groupBox2.TabIndex = 5;
			this.groupBox2.TabStop = false;
			this.groupBox2.Text = "Destination";
			// 
			// txtHost
			// 
			this.txtHost.Location = new System.Drawing.Point(32, 56);
			this.txtHost.Name = "txtHost";
			this.txtHost.Size = new System.Drawing.Size(120, 20);
			this.txtHost.TabIndex = 5;
			this.txtHost.Text = "localhost";
			// 
			// rbAddress
			// 
			this.rbAddress.Location = new System.Drawing.Point(16, 40);
			this.rbAddress.Name = "rbAddress";
			this.rbAddress.Size = new System.Drawing.Size(104, 16);
			this.rbAddress.TabIndex = 4;
			this.rbAddress.Text = "To Single Host";
			// 
			// rbBroadcast
			// 
			this.rbBroadcast.Checked = true;
			this.rbBroadcast.Location = new System.Drawing.Point(16, 16);
			this.rbBroadcast.Name = "rbBroadcast";
			this.rbBroadcast.Size = new System.Drawing.Size(104, 16);
			this.rbBroadcast.TabIndex = 3;
			this.rbBroadcast.TabStop = true;
			this.rbBroadcast.Text = "Broadcast";
			// 
			// nudPort
			// 
			this.nudPort.Location = new System.Drawing.Point(40, 24);
			this.nudPort.Maximum = new System.Decimal(new int[] {
																	65535,
																	0,
																	0,
																	0});
			this.nudPort.Minimum = new System.Decimal(new int[] {
																	1024,
																	0,
																	0,
																	0});
			this.nudPort.Name = "nudPort";
			this.nudPort.Size = new System.Drawing.Size(80, 20);
			this.nudPort.TabIndex = 0;
			this.nudPort.Value = new System.Decimal(new int[] {
																  1024,
																  0,
																  0,
																  0});
			// 
			// UDPNetworkProcessor
			// 
			this.AcceptButton = this.btApply;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(178, 199);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.groupBox2,
																		  this.groupBox1,
																		  this.btApply});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "UDPNetworkProcessor";
			this.Text = "UDP Network";
			this.groupBox1.ResumeLayout(false);
			this.groupBox2.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.nudPort)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		#region Implementation of IDatagramProcessor
		public void OnDatagram(Datagram dgram)
		{
			udpclient.Send(dgram.InternalData, dgram.InternalData.Length, destip);
		}

		private void btApply_Click(object sender, System.EventArgs e)
		{
			if(rbBroadcast.Checked)
			{
				destip = new IPEndPoint(IPAddress.Broadcast, udpport);
			}
			else
			{
				try
				{
					IPHostEntry entry = System.Net.Dns.GetHostByName(txtHost.Text);
					if(entry.AddressList.Length>0)
					{
						destip = new IPEndPoint(entry.AddressList[0], udpport);
					}
				}
				catch(Exception ex)
				{
					MessageBox.Show(this,
						"Cannot resolve hostname: \n" + ex.Message,
						"UDP Network",
						MessageBoxButtons.OK,
						MessageBoxIcon.Exclamation);
				}
			}

			udpport = (int) nudPort.Value;
			Settings.CM.SetString("Processors/" + this.InternalName + "/UDPPort", udpport.ToString());
			Settings.CM.SetString("Processors/" + this.InternalName + "/DestHost", txtHost.Text);
		}

		public string UIName
		{
			get
			{
				return "UDP Network";
			}
		}

		public string InternalName
		{
			get
			{
				return "UDPNet";
			}
		}

		public UI.BaseForm UserInterface
		{
			get
			{
				return this;
			}
		}
		#endregion
	}
}
