/*
 * VC820Tool
 * Copyright (c) 2003 by Andreas Schroeder <andreas@a-netz.de>
 *
 * This file is part of VC820Tool.
 *
 * VC820Tool is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * VC820Tool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VC820Tool; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using System;

namespace VC820Tool.Sources
{
	/// <summary>
	/// This class implements an abstract source for datagrams. 
	/// </summary>
	public abstract class AbstractDatagramSource : IDatagramSource
	{
		protected ValueDispatcher vd;
		protected bool enabled = false;

		public void SetValueDispatcher(ValueDispatcher _vd)
		{
			vd = _vd;
		}

		public override string ToString()
		{
			return UIName;
		}

		public abstract string UIName { get; }

		public abstract string InternalName { get; }

		public abstract ControllingGui UserInterface { get; }

		public bool Enabled	
		{ 
			get
			{
				return enabled;
			}
			set
			{
				enabled = value;
				if(enabled)
				{
					OnStart();
				}
				else
				{
					OnStop();
				}
			}
		}

		protected abstract void OnStart();

		protected abstract void OnStop();
	}
}
