/*
 * VC820Tool
 * Copyright (c) 2004 by Andreas Schroeder <andreas@a-netz.de>
 *
 * This file is part of VC820Tool.
 *
 * VC820Tool is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * VC820Tool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VC820Tool; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using System;
using System.Timers;

namespace VC820Tool.Sources
{
	/// <summary>
	/// This class implements a source of random datagrams to test the 
	/// program functionality
	/// </summary>
	public class DummySource : AbstractDatagramSource, IDatagramSource
	{
		private Timer timer;

		public DummySource()
		{
			timer = new Timer(500);
			timer.Elapsed += new ElapsedEventHandler(this.OnTimer);
		}

		private void OnTimer(object sender, ElapsedEventArgs e)
		{
			Datagram d;
			Random r = new Random();

			decimal val = (decimal) r.NextDouble();
			
			d = new Datagram(UnitsEnum.Volt, PrefixEnum.Kilo, val);
			
			if(vd!=null)
			{
				vd.OnNewDatagram(this, d);
			}
		}

		#region Implementation of IDatagramSource
		public override string UIName
		{
			get
			{
				return "Dummy";
			}
		}

		public override string InternalName
		{
			get
			{
				return "Dummy";
			}
		}

		public override ControllingGui UserInterface
		{
			get
			{
				return null;
			}
		}
		#endregion

		public override string ToString()
		{
			return UIName;
		}

		protected override void OnStart()
		{
			timer.Start();
		}

		protected override void OnStop()
		{
			timer.Stop();
		}
	}
}
