/*
 * VC820Tool
 * Copyright (c) 2004 by Andreas Schroeder <andreas@a-netz.de>
 *
 * This file is part of VC820Tool.
 *
 * VC820Tool is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * VC820Tool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VC820Tool; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using System;
using System.Net;
using System.Net.Sockets;
using System.Threading;

namespace VC820Tool.Sources
{
	public class UDPNetworkSource : AbstractDatagramSource, IDatagramSource
	{
		private UdpClient udpclient;
		private int udpport;
		private Thread readerthread;
		private UDPNetworkSourceCtrl ctrl;

		public UDPNetworkSource()
		{
			udpport = Settings.CM.GetInt("Source/" + this.InternalName + "/UDPPort", 5454);
			ctrl = new UDPNetworkSourceCtrl(this);
		}

		protected override void OnStart()
		{
			try
			{
				udpclient = new UdpClient(udpport);

				readerthread = new Thread(new ThreadStart(this.DoReading));
				readerthread.Name = "UDP Network Reader";
				readerthread.Start();
			}
			catch(Exception ex)
			{
				enabled = false;
				throw new Exception("Cannot instanciate UDP Client." , ex);
			}
		}

		protected override void OnStop()
		{
			readerthread.Abort();
			udpclient.Close();
		}

		private void DoReading()
		{
			IPEndPoint ip = new IPEndPoint(IPAddress.Any, 0);
			byte[] data;

			while(true)
			{
				data = udpclient.Receive(ref ip);
				if(vd!=null)
				{
					vd.OnNewDatagram(this, new Datagram(data));
				}
			}
		}

		#region Implementation of IDatagramSource
		public override string UIName
		{
			get
			{
				return "UDP Network";
			}
		}

		public override string InternalName
		{
			get
			{
				return "UDPNet";
			}
		}

		public override ControllingGui UserInterface
		{
			get
			{
				return ctrl;
			}
		}
		#endregion

		public int Port
		{
			get
			{
				return udpport;
			}
			set
			{
				udpport = value;
				readerthread.Abort();
				udpclient = new UdpClient( udpport );
				Settings.CM.SetString("Source/" + this.InternalName + "/UDPPort", udpport.ToString());

				readerthread = new Thread(new ThreadStart(this.DoReading));
				readerthread.Name = "UDP Network Reader (2?)";
				readerthread.Start();
			}
		}
	}
}
