/*
 * VC820Tool
 * Copyright (c) 2004 by Andreas Schroeder <andreas@a-netz.de>
 *
 * This file is part of VC820Tool.
 *
 * VC820Tool is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * VC820Tool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VC820Tool; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Threading;
using VC820Tool.Sources;
using SharpUtils;

namespace VC820Tool.UI
{
	public class MainForm : System.Windows.Forms.Form
	{
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.MenuItem mnuFile;
		private System.Windows.Forms.MenuItem mnuFileEnd;
		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.MenuItem mnuHelp;
		private System.Windows.Forms.MenuItem mnuHelpInfo;
		private LinkedList dispatchertrigger;
		private System.Windows.Forms.Panel pnlControl;
		private System.Windows.Forms.Splitter sp1;
		private System.Windows.Forms.Button btShow;
		private System.Windows.Forms.ListView lvProcs;
		private System.Windows.Forms.ColumnHeader chName;
		private System.Windows.Forms.CheckBox chkEnableTrigger;
		private System.Windows.Forms.Panel pnlTriggerControl;
		private System.Windows.Forms.ComboBox cmbTrigger;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.GroupBox groupBox2;
		private ValueDispatcher dispatcher;
		private VDTrigger.AbstractVDTrigger activetrigger = null;
		private System.Windows.Forms.GroupBox groupBox3;
		private System.Windows.Forms.ComboBox cmbSources;
		private System.Windows.Forms.Panel pnlSourceControl;
		private LinkedList dgprocs;
		private System.Windows.Forms.MainMenu mainMenu;
		private System.Windows.Forms.GroupBox groupBox4;
		private System.Windows.Forms.Button btResetWarn;
		private System.Windows.Forms.Label laWarn;
		private System.Windows.Forms.Button btSourceApply;
		private System.Windows.Forms.Button btSourceReset;
		private System.Windows.Forms.Button btTriggerApply;
		private System.Windows.Forms.Button btTriggerReset;
		private DataFlasher pnlFlasher;
		private IDatagramSource activesource = null;

		public MainForm()
		{
			SplashScreen splash = new SplashScreen();
			splash.ShowSplash(this, 3000);

			InitializeComponent();

			this.Text = Settings.AppTitle;
			this.WindowState = (FormWindowState) Settings.CM.GetInt("Windows/Main/WindowState", 
				(int)FormWindowState.Normal);
			Size size = new Size(
				Settings.CM.GetInt("/Windows/Main/Width", 600),
				Settings.CM.GetInt("/Windows/Main/Height", 400));
			this.Size = size;

			InitializeCoreComponents();
		}

		/// <summary>
		/// Initialize the core components of the application
		/// </summary>
		public void InitializeCoreComponents()
		{
			// create the one and only value dispatcher
			dispatcher = new ValueDispatcher();
			dispatcher.NewDatagram += new NewDatagramHandler(this.OnNewDatagram);
			dispatcher.DatagramLost+= new EventHandler(dispatcher_DatagramLost);

			// create the datagram sources
			InitializeDatagramSources();

			// create the dispatcher triggers
			InitializeDispatcherTrigger();
			
			// initialize the datagram processors
			InitializeDatagramProcessors();
		}

		/// <summary>
		/// create all possible datagram sources
		/// </summary>
		private void InitializeDatagramSources()
		{
			RegisterSource( new Sources.SerialSource() );
			RegisterSource( new Sources.DummySource() );
			RegisterSource( new Sources.UDPNetworkSource() );
		}

		/// <summary>
		/// Create the dispatcher triggers
		/// </summary>
		private void InitializeDispatcherTrigger()
		{
			dispatchertrigger = new LinkedList();

			RegisterDispatcherTrigger(new VDTrigger.SingleShotVDTrigger());
			
			RegisterDispatcherTrigger(new VDTrigger.TimerVDTrigger());

			VDTrigger.InstrumentVDTrigger t = new VDTrigger.InstrumentVDTrigger();
			dispatcher.NewDatagram += new NewDatagramHandler( t.OnNewDatagram );
			RegisterDispatcherTrigger(t);
		}

		/// <summary>
		/// Create the datagram processors
		/// </summary>
		private void InitializeDatagramProcessors()
		{
			dgprocs = new LinkedList();	
			
			RegisterDatagramProcessor( new Processors.TextOutputForm() );
			RegisterDatagramProcessor( new Processors.FileRecorderForm() );
			RegisterDatagramProcessor( new Processors.GraphForm() );
			RegisterDatagramProcessor( new Processors.StatisticsForm() );
			RegisterDatagramProcessor( new Processors.UDPNetworkProcessor() );
			//RegisterDatagramProcessor( new Processors.DeltaPeakCharger() );
			RegisterDatagramProcessor( new Processors.DataList() );
		}

		private void RegisterSource(IDatagramSource source)
		{
			cmbSources.Items.Add(source);
		}

		private void RegisterDispatcherTrigger(VDTrigger.AbstractVDTrigger trigger)
		{
			trigger.Dispatcher = dispatcher;
			dispatchertrigger.Add(trigger);
			cmbTrigger.Items.Add(trigger);
		}

		private void RegisterDatagramProcessor(IDatagramProcessor dgproc)
		{
			dgprocs.Add(dgproc);

			dgproc.UserInterface.MdiParent = this;
			LoadProcWindowPos(dgproc);

			ListViewItem lvitem = lvProcs.Items.Add(dgproc.UIName);
			lvitem.Tag = dgproc;
			lvitem.Checked = false;
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(MainForm));
			this.mainMenu = new System.Windows.Forms.MainMenu();
			this.mnuFile = new System.Windows.Forms.MenuItem();
			this.mnuFileEnd = new System.Windows.Forms.MenuItem();
			this.menuItem1 = new System.Windows.Forms.MenuItem();
			this.mnuHelp = new System.Windows.Forms.MenuItem();
			this.mnuHelpInfo = new System.Windows.Forms.MenuItem();
			this.pnlControl = new System.Windows.Forms.Panel();
			this.groupBox4 = new System.Windows.Forms.GroupBox();
			this.btResetWarn = new System.Windows.Forms.Button();
			this.laWarn = new System.Windows.Forms.Label();
			this.groupBox3 = new System.Windows.Forms.GroupBox();
			this.btSourceApply = new System.Windows.Forms.Button();
			this.btSourceReset = new System.Windows.Forms.Button();
			this.pnlSourceControl = new System.Windows.Forms.Panel();
			this.cmbSources = new System.Windows.Forms.ComboBox();
			this.groupBox2 = new System.Windows.Forms.GroupBox();
			this.btShow = new System.Windows.Forms.Button();
			this.lvProcs = new System.Windows.Forms.ListView();
			this.chName = new System.Windows.Forms.ColumnHeader();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.pnlTriggerControl = new System.Windows.Forms.Panel();
			this.cmbTrigger = new System.Windows.Forms.ComboBox();
			this.chkEnableTrigger = new System.Windows.Forms.CheckBox();
			this.btTriggerApply = new System.Windows.Forms.Button();
			this.btTriggerReset = new System.Windows.Forms.Button();
			this.sp1 = new System.Windows.Forms.Splitter();
			this.pnlFlasher = new DataFlasher();
			this.pnlControl.SuspendLayout();
			this.groupBox4.SuspendLayout();
			this.groupBox3.SuspendLayout();
			this.groupBox2.SuspendLayout();
			this.groupBox1.SuspendLayout();
			this.SuspendLayout();
			// 
			// mainMenu
			// 
			this.mainMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.mnuFile,
																					 this.menuItem1,
																					 this.mnuHelp});
			// 
			// mnuFile
			// 
			this.mnuFile.Index = 0;
			this.mnuFile.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					this.mnuFileEnd});
			this.mnuFile.Text = "&File";
			// 
			// mnuFileEnd
			// 
			this.mnuFileEnd.Index = 0;
			this.mnuFileEnd.Text = "E&xit";
			this.mnuFileEnd.Click += new System.EventHandler(this.mnuFileEnd_Click);
			// 
			// menuItem1
			// 
			this.menuItem1.Index = 1;
			this.menuItem1.MdiList = true;
			this.menuItem1.Text = "&Window";
			// 
			// mnuHelp
			// 
			this.mnuHelp.Index = 2;
			this.mnuHelp.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					this.mnuHelpInfo});
			this.mnuHelp.Text = "&Help";
			// 
			// mnuHelpInfo
			// 
			this.mnuHelpInfo.Index = 0;
			this.mnuHelpInfo.Text = "&About ...";
			this.mnuHelpInfo.Click += new System.EventHandler(this.mnuHelpInfo_Click);
			// 
			// pnlControl
			// 
			this.pnlControl.Controls.Add(this.groupBox4);
			this.pnlControl.Controls.Add(this.groupBox3);
			this.pnlControl.Controls.Add(this.groupBox2);
			this.pnlControl.Controls.Add(this.groupBox1);
			this.pnlControl.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.pnlControl.Location = new System.Drawing.Point(0, 277);
			this.pnlControl.Name = "pnlControl";
			this.pnlControl.Size = new System.Drawing.Size(816, 160);
			this.pnlControl.TabIndex = 6;
			// 
			// groupBox4
			// 
			this.groupBox4.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left)));
			this.groupBox4.Controls.Add(this.btResetWarn);
			this.groupBox4.Controls.Add(this.laWarn);
			this.groupBox4.Location = new System.Drawing.Point(656, 8);
			this.groupBox4.Name = "groupBox4";
			this.groupBox4.Size = new System.Drawing.Size(152, 144);
			this.groupBox4.TabIndex = 29;
			this.groupBox4.TabStop = false;
			this.groupBox4.Text = "Warnings";
			// 
			// btResetWarn
			// 
			this.btResetWarn.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.btResetWarn.Location = new System.Drawing.Point(88, 120);
			this.btResetWarn.Name = "btResetWarn";
			this.btResetWarn.Size = new System.Drawing.Size(56, 20);
			this.btResetWarn.TabIndex = 32;
			this.btResetWarn.Text = "Reset";
			this.btResetWarn.Click += new System.EventHandler(this.btResetWarn_Click);
			// 
			// laWarn
			// 
			this.laWarn.Location = new System.Drawing.Point(8, 16);
			this.laWarn.Name = "laWarn";
			this.laWarn.Size = new System.Drawing.Size(136, 96);
			this.laWarn.TabIndex = 31;
			// 
			// groupBox3
			// 
			this.groupBox3.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left)));
			this.groupBox3.Controls.Add(this.pnlFlasher);
			this.groupBox3.Controls.Add(this.btSourceApply);
			this.groupBox3.Controls.Add(this.btSourceReset);
			this.groupBox3.Controls.Add(this.pnlSourceControl);
			this.groupBox3.Controls.Add(this.cmbSources);
			this.groupBox3.Location = new System.Drawing.Point(8, 8);
			this.groupBox3.Name = "groupBox3";
			this.groupBox3.Size = new System.Drawing.Size(208, 144);
			this.groupBox3.TabIndex = 28;
			this.groupBox3.TabStop = false;
			this.groupBox3.Text = "Source";
			// 
			// btSourceApply
			// 
			this.btSourceApply.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.btSourceApply.Location = new System.Drawing.Point(80, 120);
			this.btSourceApply.Name = "btSourceApply";
			this.btSourceApply.Size = new System.Drawing.Size(56, 20);
			this.btSourceApply.TabIndex = 3;
			this.btSourceApply.Text = "Apply";
			this.btSourceApply.Click += new System.EventHandler(this.btSourceApply_Click);
			// 
			// btSourceReset
			// 
			this.btSourceReset.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.btSourceReset.Location = new System.Drawing.Point(144, 120);
			this.btSourceReset.Name = "btSourceReset";
			this.btSourceReset.Size = new System.Drawing.Size(56, 20);
			this.btSourceReset.TabIndex = 2;
			this.btSourceReset.Text = "Reset";
			this.btSourceReset.Click += new System.EventHandler(this.btSourceReset_Click);
			// 
			// pnlSourceControl
			// 
			this.pnlSourceControl.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.pnlSourceControl.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.pnlSourceControl.Location = new System.Drawing.Point(8, 40);
			this.pnlSourceControl.Name = "pnlSourceControl";
			this.pnlSourceControl.Size = new System.Drawing.Size(192, 72);
			this.pnlSourceControl.TabIndex = 1;
			// 
			// cmbSources
			// 
			this.cmbSources.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.cmbSources.Location = new System.Drawing.Point(8, 16);
			this.cmbSources.Name = "cmbSources";
			this.cmbSources.Size = new System.Drawing.Size(192, 21);
			this.cmbSources.TabIndex = 0;
			this.cmbSources.SelectedValueChanged += new System.EventHandler(this.cmbSources_SelectedValueChanged);
			// 
			// groupBox2
			// 
			this.groupBox2.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left)));
			this.groupBox2.Controls.Add(this.btShow);
			this.groupBox2.Controls.Add(this.lvProcs);
			this.groupBox2.Location = new System.Drawing.Point(440, 8);
			this.groupBox2.Name = "groupBox2";
			this.groupBox2.Size = new System.Drawing.Size(208, 144);
			this.groupBox2.TabIndex = 27;
			this.groupBox2.TabStop = false;
			this.groupBox2.Text = "Processors";
			// 
			// btShow
			// 
			this.btShow.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.btShow.Location = new System.Drawing.Point(144, 120);
			this.btShow.Name = "btShow";
			this.btShow.Size = new System.Drawing.Size(56, 20);
			this.btShow.TabIndex = 20;
			this.btShow.Text = "Show";
			this.btShow.Click += new System.EventHandler(this.OnShowProcUI);
			// 
			// lvProcs
			// 
			this.lvProcs.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.lvProcs.CheckBoxes = true;
			this.lvProcs.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																					  this.chName});
			this.lvProcs.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.lvProcs.HideSelection = false;
			this.lvProcs.Location = new System.Drawing.Point(8, 16);
			this.lvProcs.Name = "lvProcs";
			this.lvProcs.Size = new System.Drawing.Size(192, 96);
			this.lvProcs.Sorting = System.Windows.Forms.SortOrder.Ascending;
			this.lvProcs.TabIndex = 18;
			this.lvProcs.View = System.Windows.Forms.View.Details;
			this.lvProcs.DoubleClick += new System.EventHandler(this.OnShowProcUI);
			this.lvProcs.ItemCheck += new System.Windows.Forms.ItemCheckEventHandler(this.lvProcs_ItemCheck);
			// 
			// chName
			// 
			this.chName.Text = "Name";
			this.chName.Width = 160;
			// 
			// groupBox1
			// 
			this.groupBox1.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left)));
			this.groupBox1.Controls.Add(this.pnlTriggerControl);
			this.groupBox1.Controls.Add(this.cmbTrigger);
			this.groupBox1.Controls.Add(this.chkEnableTrigger);
			this.groupBox1.Controls.Add(this.btTriggerApply);
			this.groupBox1.Controls.Add(this.btTriggerReset);
			this.groupBox1.Location = new System.Drawing.Point(224, 8);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(208, 144);
			this.groupBox1.TabIndex = 26;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Trigger";
			// 
			// pnlTriggerControl
			// 
			this.pnlTriggerControl.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.pnlTriggerControl.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.pnlTriggerControl.Location = new System.Drawing.Point(8, 40);
			this.pnlTriggerControl.Name = "pnlTriggerControl";
			this.pnlTriggerControl.Size = new System.Drawing.Size(192, 72);
			this.pnlTriggerControl.TabIndex = 23;
			// 
			// cmbTrigger
			// 
			this.cmbTrigger.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.cmbTrigger.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.cmbTrigger.Location = new System.Drawing.Point(8, 16);
			this.cmbTrigger.Name = "cmbTrigger";
			this.cmbTrigger.Size = new System.Drawing.Size(192, 21);
			this.cmbTrigger.TabIndex = 21;
			this.cmbTrigger.SelectedValueChanged += new System.EventHandler(this.cmbTrigger_SelectedValueChanged);
			// 
			// chkEnableTrigger
			// 
			this.chkEnableTrigger.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.chkEnableTrigger.Location = new System.Drawing.Point(8, 120);
			this.chkEnableTrigger.Name = "chkEnableTrigger";
			this.chkEnableTrigger.Size = new System.Drawing.Size(64, 16);
			this.chkEnableTrigger.TabIndex = 24;
			this.chkEnableTrigger.Text = "Enable";
			this.chkEnableTrigger.CheckedChanged += new System.EventHandler(this.chkEnableTrigger_CheckedChanged);
			// 
			// btTriggerApply
			// 
			this.btTriggerApply.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.btTriggerApply.Location = new System.Drawing.Point(80, 120);
			this.btTriggerApply.Name = "btTriggerApply";
			this.btTriggerApply.Size = new System.Drawing.Size(56, 20);
			this.btTriggerApply.TabIndex = 3;
			this.btTriggerApply.Text = "Apply";
			this.btTriggerApply.Click += new System.EventHandler(this.btTriggerApply_Click);
			// 
			// btTriggerReset
			// 
			this.btTriggerReset.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.btTriggerReset.Location = new System.Drawing.Point(144, 120);
			this.btTriggerReset.Name = "btTriggerReset";
			this.btTriggerReset.Size = new System.Drawing.Size(56, 20);
			this.btTriggerReset.TabIndex = 2;
			this.btTriggerReset.Text = "Reset";
			this.btTriggerReset.Click += new System.EventHandler(this.btTriggerReset_Click);
			// 
			// sp1
			// 
			this.sp1.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.sp1.Location = new System.Drawing.Point(0, 274);
			this.sp1.MinSize = 125;
			this.sp1.Name = "sp1";
			this.sp1.Size = new System.Drawing.Size(816, 3);
			this.sp1.TabIndex = 7;
			this.sp1.TabStop = false;
			// 
			// pnlFlasher
			// 
			this.pnlFlasher.Location = new System.Drawing.Point(8, 120);
			this.pnlFlasher.Name = "pnlFlasher";
			this.pnlFlasher.Size = new System.Drawing.Size(16, 16);
			this.pnlFlasher.TabIndex = 4;
			// 
			// MainForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(816, 437);
			this.Controls.Add(this.sp1);
			this.Controls.Add(this.pnlControl);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.IsMdiContainer = true;
			this.Menu = this.mainMenu;
			this.MinimumSize = new System.Drawing.Size(632, 464);
			this.Name = "MainForm";
			this.StartPosition = System.Windows.Forms.FormStartPosition.Manual;
			this.Closing += new System.ComponentModel.CancelEventHandler(this.MainForm_Closing);
			this.pnlControl.ResumeLayout(false);
			this.groupBox4.ResumeLayout(false);
			this.groupBox3.ResumeLayout(false);
			this.groupBox2.ResumeLayout(false);
			this.groupBox1.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		private void mnuFileEnd_Click(object sender, System.EventArgs e)
		{
			OnShutdown();

			Application.Exit();
		}

		/// <summary>
		/// This should be called on shutdown
		/// </summary>
		private void OnShutdown() 
		{
			// shutdown active datagram source
			if(activesource != null && activesource.Enabled)
			{
				activesource.Enabled = false;
			}

			// save the state of each datagram processor
			foreach(IDatagramProcessor dgproc in dgprocs)
			{
				SaveProcWindowPos(dgproc);
			}

			Settings.CM.SetString("Windows/Main/WindowState", ((int)this.WindowState).ToString());
			Settings.CM.SetString("Windows/Main/Width", this.Size.Width.ToString());
			Settings.CM.SetString("Windows/Main/Height", this.Size.Height.ToString());

			try
			{
				Settings.CM.Save();
			}
			catch(Exception ex)
			{
				MessageBox.Show(this, "Error while writing configuration file '" 
					+ Settings.CM.Filename + "':\n" + ex.Message, 
					"VC820 Tool", 
					MessageBoxButtons.OK, 
					MessageBoxIcon.Exclamation);
			}
		}

		private void MainForm_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			OnShutdown();

			Application.Exit();
		}

		/// <summary>
		/// Save the window position and size of every processor
		/// </summary>
		/// <param name="dgproc"></param>
		private void SaveProcWindowPos(IDatagramProcessor dgproc)
		{
			Settings.CM.SetString("Processor/" + dgproc.InternalName + "/UI/X", dgproc.UserInterface.Location.X.ToString());
			Settings.CM.SetString("Processor/" + dgproc.InternalName + "/UI/Y", dgproc.UserInterface.Location.Y.ToString());
			Settings.CM.SetString("Processor/" + dgproc.InternalName + "/UI/Width", dgproc.UserInterface.Width.ToString());
			Settings.CM.SetString("Processor/" + dgproc.InternalName + "/UI/Height", dgproc.UserInterface.Height.ToString());
			Settings.CM.SetString("Processor/" + dgproc.InternalName + "/UI/Visible", dgproc.UserInterface.Visible.ToString());
		}

		/// <summary>
		/// Load the window position and size of a datagram processor
		/// </summary>
		/// <param name="dgproc"></param>
		private void LoadProcWindowPos(IDatagramProcessor dgproc)
		{
			Point loc = new Point(
				Settings.CM.GetInt("Processor/" + dgproc.InternalName + "/UI/X", 0),
				Settings.CM.GetInt("Processor/" + dgproc.InternalName + "/UI/Y", 0));
			dgproc.UserInterface.Location = loc;

			if(Settings.CM.IsSet("Processor/" + dgproc.InternalName + "/UI/Width"))
			{
				dgproc.UserInterface.Width = Settings.CM.GetInt("Processor/" + dgproc.InternalName + "/UI/Width", 100);
				dgproc.UserInterface.Height = Settings.CM.GetInt("Processor/" + dgproc.InternalName + "/UI/Height", 100);
			}

			dgproc.UserInterface.Visible = 
				Settings.CM.GetBool("Processor/" + dgproc.InternalName + "/UI/Visible", true);
		}

		private void mnuHelpInfo_Click(object sender, System.EventArgs e)
		{
			new InfoDialog().ShowDialog(this);
		}

		public VDTrigger.AbstractVDTrigger ActiveTrigger
		{
			get
			{
				return activetrigger;
			}
			set
			{
				if(activetrigger == value)
				{
					return;
				}
				
				// disable current trigger
				if(activetrigger != null)
				{
					activetrigger.Enable = false;
				}

				// set up the new trigger
				activetrigger = value;

				pnlTriggerControl.Controls.Clear();
			
				if(activetrigger != null)
				{
					activetrigger.Enable = chkEnableTrigger.Checked;

					if(activetrigger.UserInterface != null)
					{
						pnlTriggerControl.Controls.Add(activetrigger.UserInterface);
						activetrigger.UserInterface.Size = pnlTriggerControl.ClientSize;
						activetrigger.UserInterface.Dock = DockStyle.Fill;
					}
				}
			}
		}

		private void cmbTrigger_SelectedValueChanged(object sender, System.EventArgs e)
		{
			ActiveTrigger = (VDTrigger.AbstractVDTrigger) cmbTrigger.SelectedItem;
		}

		/// <summary>
		/// Changes the activation state of a datagram processor
		/// </summary>
		/// <param name="processor"></param>
		/// <param name="enable"></param>
		private void SetProcessorState(IDatagramProcessor processor, bool enable)
		{
			if(enable)
			{
				dispatcher.AddDatagramProcessor(processor);
			}
			else
			{
				dispatcher.RemoveDatagramProcessor(processor);
			}
		}

		private void lvProcs_ItemCheck(object sender, System.Windows.Forms.ItemCheckEventArgs e)
		{
			IDatagramProcessor proc = (IDatagramProcessor) lvProcs.Items[e.Index].Tag;

			SetProcessorState(proc, (e.NewValue == CheckState.Checked) );
		}

		private void OnShowProcUI(object sender, System.EventArgs e)
		{
			IDatagramProcessor proc;

			for(int i=0; i < lvProcs.SelectedItems.Count; ++i)
			{
				proc = (IDatagramProcessor) lvProcs.SelectedItems[i].Tag;
				ShowProccessorUI(proc);
			}
		}

		/// <summary>
		/// Show the user interface of a datagram processor
		/// </summary>
		/// <param name="processor"></param>
		private void ShowProccessorUI(IDatagramProcessor processor)
		{
			processor.UserInterface.Show();
			processor.UserInterface.Activate();
		}

		private void chkEnableTrigger_CheckedChanged(object sender, System.EventArgs e)
		{
			if(ActiveTrigger != null)
			{
				ActiveTrigger.Enable = chkEnableTrigger.Checked;
			}
		}

		private IDatagramSource ActiveSource
		{
			get
			{
				return activesource;
			}
			set
			{
				if(activesource == value)
				{
					return;
				}

				// release old source
				if(activesource != null)
				{
					activesource.Enabled = false;
					activesource.SetValueDispatcher(null);
				}

				activesource = value;

				// set up new source
				pnlSourceControl.Controls.Clear();
				if(activesource != null)
				{
					if(activesource.UserInterface != null)
					{
						activesource.UserInterface.Dock = DockStyle.Fill;
						pnlSourceControl.Controls.Add(activesource.UserInterface);
					}

					activesource.SetValueDispatcher(dispatcher);
					try
					{
						activesource.Enabled = true;
					} 
					catch(Exception ex)
					{
						MessageBox.Show(this, "Cannot activate selected source: " + ex.Message);
						ActiveSource = null;
					}
				}
			}
		}


		private void cmbSources_SelectedValueChanged(object sender, System.EventArgs e)
		{
			ActiveSource = (IDatagramSource) cmbSources.SelectedItem;
		}

		/// <summary>
		/// Applies the settings made in the source control
		/// to the source itself
		/// </summary>
		private void ApplySourceSettings() 
		{
			try
			{
				if(ActiveSource != null && ActiveSource.UserInterface != null)
				{
					ActiveSource.UserInterface.ApplySettings();
				}
			}
			catch(Exception ex)
			{
				MessageBox.Show(this,
					"Exception while applying settings to source: \n"
					+ ex.Message,
					Settings.AppTitle,
					MessageBoxButtons.OK,
					MessageBoxIcon.Exclamation);
			}
		}

		/// <summary>
		/// Reset the values in the source control to the values
		/// of the source ifself
		/// </summary>
		private void ResetSourceSettings()
		{
			try
			{
				if(ActiveSource != null && ActiveSource.UserInterface != null)
				{
					ActiveSource.UserInterface.ResetSettings();
				}
			}
			catch(Exception ex)
			{
				MessageBox.Show(this,
					"Exception while resetting settings of source: \n"
					+ ex.Message,
					Settings.AppTitle,
					MessageBoxButtons.OK,
					MessageBoxIcon.Exclamation);
			}

		}

		private void ApplyTriggerSettings() 
		{
			try
			{
				if(ActiveTrigger != null && ActiveTrigger.UserInterface != null)
				{
					ActiveTrigger.UserInterface.ApplySettings();
				}
			}
			catch(Exception ex)
			{
				MessageBox.Show(this,
					"Exception while applying trigger settings: \n"
					+ ex.Message,
					Settings.AppTitle,
					MessageBoxButtons.OK,
					MessageBoxIcon.Exclamation);
			}
		}

		private void ResetTriggerSettings()
		{
			try
			{
				if(ActiveTrigger != null && ActiveTrigger.UserInterface != null)
				{
					ActiveTrigger.UserInterface.ResetSettings();
				}
			}
			catch(Exception ex)
			{
				MessageBox.Show(this,
					"Exception while resetting settings of source: \n"
					+ ex.Message,
					Settings.AppTitle,
					MessageBoxButtons.OK,
					MessageBoxIcon.Exclamation);
			}

		}

		private void btSourceApply_Click(object sender, System.EventArgs e)
		{
			ApplySourceSettings();
		}

		private void btSourceReset_Click(object sender, System.EventArgs e)
		{
			ResetSourceSettings();
		}

		private void OnNewDatagram(IDatagramSource source, Datagram dgram)
		{
			pnlFlasher.Flash();
		}

		private void dispatcher_DatagramLost(object sender, EventArgs e)
		{
			laWarn.Text = "There are datagrams lost! Decrease processing load.";
		}

		private void btResetWarn_Click(object sender, System.EventArgs e)
		{
			laWarn.Text = "";
		}

		private void btTriggerApply_Click(object sender, System.EventArgs e)
		{
			ApplyTriggerSettings();
		}

		private void btTriggerReset_Click(object sender, System.EventArgs e)
		{
			ResetTriggerSettings();
		}
	}
}
