/*
 * VC820Tool
 * Copyright (c) 2004 by Andreas Schroeder <andreas@a-netz.de>
 *
 * This file is part of VC820Tool.
 *
 * VC820Tool is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * VC820Tool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VC820Tool; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Threading;

namespace VC820Tool.UI
{
	/// <summary>
	/// Summary description for SplashScreen.
	/// </summary>
	public class SplashScreen : System.Windows.Forms.Form
	{
		private int duration = 3000;
		private Form parent;
		private System.Windows.Forms.Panel panel3;
		private System.Windows.Forms.Panel panel2;
		private System.Windows.Forms.Panel panel1;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Label laAppVersion;
		private System.Windows.Forms.Label laAppTitle;
		private System.Timers.Timer timer;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public SplashScreen()
		{
			StartPosition = FormStartPosition.CenterScreen;

			InitializeComponent();

			laAppTitle.Text = Settings.AppTitle;
			laAppVersion.Text = "Version " + Settings.AppVersion;
		}

		public void ShowSplash(Form parent, int time) 
		{
			duration = time;
			this.parent = parent;

			timer = new System.Timers.Timer();
			timer.AutoReset = false;
			timer.Interval = (double) time;
			timer.Elapsed += new System.Timers.ElapsedEventHandler(timer_Elapsed);
			timer.Start();

			Thread splash = new Thread(new ThreadStart(Splash));
			splash.Name = "Splash Screen";
			splash.Start();
		}

		private void Splash()
		{
			Application.Run(this);
		}

		public void CloseSplash()
		{
			parent.Activate();
			Close();
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.panel3 = new System.Windows.Forms.Panel();
			this.panel2 = new System.Windows.Forms.Panel();
			this.panel1 = new System.Windows.Forms.Panel();
			this.label3 = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.laAppVersion = new System.Windows.Forms.Label();
			this.laAppTitle = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// panel3
			// 
			this.panel3.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.panel3.BackColor = System.Drawing.Color.Firebrick;
			this.panel3.Location = new System.Drawing.Point(72, 56);
			this.panel3.Name = "panel3";
			this.panel3.Size = new System.Drawing.Size(288, 8);
			this.panel3.TabIndex = 17;
			// 
			// panel2
			// 
			this.panel2.BackColor = System.Drawing.Color.Firebrick;
			this.panel2.Location = new System.Drawing.Point(48, 56);
			this.panel2.Name = "panel2";
			this.panel2.Size = new System.Drawing.Size(16, 8);
			this.panel2.TabIndex = 16;
			// 
			// panel1
			// 
			this.panel1.BackColor = System.Drawing.Color.Firebrick;
			this.panel1.Location = new System.Drawing.Point(8, 56);
			this.panel1.Name = "panel1";
			this.panel1.Size = new System.Drawing.Size(32, 8);
			this.panel1.TabIndex = 15;
			// 
			// label3
			// 
			this.label3.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.label3.Font = new System.Drawing.Font("Arial", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label3.Location = new System.Drawing.Point(16, 180);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(334, 16);
			this.label3.TabIndex = 14;
			this.label3.Text = "see Help - Info for licence details";
			this.label3.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// label2
			// 
			this.label2.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.label2.Font = new System.Drawing.Font("Arial", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label2.Location = new System.Drawing.Point(16, 116);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(334, 16);
			this.label2.TabIndex = 13;
			this.label2.Text = "(c) 2004 by Andreas Schrder <andreas@a-netz.de>";
			this.label2.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// laAppVersion
			// 
			this.laAppVersion.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.laAppVersion.Font = new System.Drawing.Font("Arial", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.laAppVersion.Location = new System.Drawing.Point(16, 84);
			this.laAppVersion.Name = "laAppVersion";
			this.laAppVersion.Size = new System.Drawing.Size(334, 16);
			this.laAppVersion.TabIndex = 12;
			this.laAppVersion.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// laAppTitle
			// 
			this.laAppTitle.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.laAppTitle.Font = new System.Drawing.Font("Palatino Linotype", 20.25F, System.Drawing.FontStyle.Italic, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.laAppTitle.Location = new System.Drawing.Point(16, 12);
			this.laAppTitle.Name = "laAppTitle";
			this.laAppTitle.Size = new System.Drawing.Size(334, 44);
			this.laAppTitle.TabIndex = 11;
			this.laAppTitle.Text = "VC820 Measurement Tool";
			this.laAppTitle.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// SplashScreen
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(368, 208);
			this.Controls.Add(this.panel3);
			this.Controls.Add(this.panel2);
			this.Controls.Add(this.panel1);
			this.Controls.Add(this.label3);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.laAppVersion);
			this.Controls.Add(this.laAppTitle);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
			this.Name = "SplashScreen";
			this.Text = "SplashScreen";
			this.Click += new System.EventHandler(this.SplashScreen_Click);
			this.ResumeLayout(false);

		}
		#endregion


		private void SplashScreen_Click(object sender, System.EventArgs e)
		{
			CloseSplash();
		}

		private void timer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
		{
			CloseSplash();
		}
	}
}
