/*
 * VC820Tool
 * Copyright (c) 2004 by Andreas Schroeder <andreas@a-netz.de>
 *
 * This file is part of VC820Tool.
 *
 * VC820Tool is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * VC820Tool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VC820Tool; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using System;
using System.Windows.Forms;

namespace VC820Tool.VDTrigger
{
	/// <summary>
	/// A VDTrigger (ValueDispatcher Trigger) notifies the datagram processors
	/// when to process the next datagramm. This might happen if a new datagram
	/// is read from the instrument or controlled by a timer, ...
	/// </summary>
	public abstract class AbstractVDTrigger : IDispatcherTrigger
	{
		protected ValueDispatcher vd;
		protected ControllingGui ctrl = null;
		protected bool m_enabled = false;

		public ValueDispatcher Dispatcher
		{
			get
			{
				return vd;
			}
			set
			{
				vd = value;
			}
		}

		public abstract string Name { get;}

		public override string ToString()
		{
			return Name;
		}

		public ControllingGui UserInterface
		{
			get
			{
				return ctrl;
			}
		}

		public abstract bool Enable
		{
			get; set;
		}

//		public abstract void ApplySettings();
//
//		public abstract void ResetSettings();
	}
}
